@@ -5,6 +5,9 @@ use parent qw( Plack::Component );
 use RDF::LinkedData;
 use URI::NamespaceMap;
 use Plack::Request;
+use Try::Tiny;
+use Carp;
+use Module::Load::Conditional qw[can_load];
 
 =head1 NAME
 
@@ -12,11 +15,11 @@ Plack::App::RDF::LinkedData - A Plack application for running RDF::LinkedData
 
 =head1 VERSION
 
-Version 1.01_01
+Version 1.90_01
 
 =cut
 
- our $VERSION = '1.01_01';
+ our $VERSION = '1.90_01';
 
 
 =head1 SYNOPSIS
@@ -53,6 +56,11 @@ This will start a server with the default config on localhost on port
 5000, so the URIs you're going serve from the file data.ttl will have
 to have a base URI C<http://localhost:5000/>.
 
+There is also a C<LOG_ADAPTER> that can be set to any of
+L<Log::Any::Adapter> to send logging to the console. If used with
+L<Log::Any::Adapter::Screen>, several other environment variables can
+be used to further control it.
+
 =head3 Using perlrdf command line tool
 
 A slightly longer example requires L<App::perlrdf>, but sets up a
@@ -130,8 +138,8 @@ work-in-progress, It is a more lightweight but less powerful way to
 query RDF data than SPARQL. If you have this, it is recommended to
 have CORS enabled and required to have at least a minimal VoID setup.
 
-Note that in some environments, for example if the Plack server that
-is dynamically configured, and/or behind a proxy server, the server
+Note that in some environments, for example if the Plack server
+is dynamically configured and/or behind a proxy server, the server
 may fail to bind to the address you give it as hostname. In this case,
 it is wise to allow the server to bind to any public IP address,
 i.e. set the host name to 0.0.0.0.
@@ -317,12 +325,20 @@ Will be called by Plack to process the request.
 sub prepare_app {
 	my $self = shift;
 	my $config = $self->{config};
-	$self->{linkeddata} = RDF::LinkedData->new(store => $config->{store},
-															 endpoint_config => $config->{endpoint},
-															 void_config => $config->{void},
-															 fragments_config => $config->{fragments},
-															 base_uri => $config->{base_uri}
-															);
+	if (defined $config->{'class'}) {
+	  my $class = $config->{'class'};
+	  unless (can_load( modules => { $class => 0 })) {
+		 croak "Configured $class cannot be loaded, is it installed?";
+	  }
+	  try {
+		 $self->{linkeddata} = $class->new($config);
+	  } catch {
+		 croak "Application cannot use $class as configured.";
+	  };
+	  croak "Configured $class not a subclass of RDF::LinkedData" unless ($self->{linkeddata}->isa('RDF::LinkedData'));
+	} else {
+	  $self->{linkeddata} = RDF::LinkedData->new($config);
+	}
 	$self->{linkeddata}->namespaces(URI::NamespaceMap->new($config->{namespaces})) if ($config->{namespaces});
 	# Ensure that certain namespaces are always declared
 	$self->{linkeddata}->guess_namespaces('rdf', 'dc', 'xsd', 'void');
@@ -334,12 +350,11 @@ sub call {
 	my $req = Plack::Request->new($env);
 	my $uri = $req->uri;
 	my $ld = $self->{linkeddata};
-	my $endpoint_path;
-	if ($ld->has_endpoint) {
-	  $endpoint_path = $ld->endpoint_config->{endpoint_path};
-	}
-	unless (($req->method eq 'GET') || ($req->method eq 'HEAD')
-			  || (($req->method eq 'POST') && defined($endpoint_path) && ($uri =~ m|$endpoint_path$|))) {
+
+	# Never return 405 here if writes are enabled by config, only do it if there isn't a read operation and writes are not enabled
+	my $does_read_operation = $self->does_read_operation($req);
+	$ld->does_read_operation($does_read_operation);
+	unless ($does_read_operation || $self->{config}->{writes_enabled}) {
 		return [ 405, [ 'Content-type', 'text/plain' ], [ 'Method not allowed' ] ];
 	}
 
@@ -347,14 +362,34 @@ sub call {
 		return [ 302, [ 'Location', $ld->base_uri . '/' ], [ '' ] ];
 	}
 
-	if ($uri->as_iri =~ m!^(.+?)/?(page|data)$!) {
-		$uri = URI->new($1);
-		$ld->type($2);
+	if ($uri->as_iri =~ m!^(.+?)/?(page|data|controls)$!) {
+	  $uri = URI->new($1);
+	  $ld->type($2);
+	} else {
+	  $ld->type('');
 	}
 	$ld->request($req);
 	return $ld->response($uri)->finalize;
 }
 
+sub auth_required {
+	my ($self, $req) = @_;
+	return ($self->{config}->{writes_enabled} && (! $self->does_read_operation($req)));
+
+}
+
+sub does_read_operation {
+	my ($self, $req) = @_;
+	my $uri = $req->uri;
+	my $endpoint_path;
+	my $ld = $self->{linkeddata}; # Might be a performance problem
+	if ($ld->has_endpoint) {
+	  $endpoint_path = $ld->endpoint_config->{endpoint_path};
+	}
+	return (($req->method eq 'GET') || ($req->method eq 'HEAD')
+			  || (($req->method eq 'POST') && defined($endpoint_path) && ($uri =~ m|$endpoint_path$|)))
+}
+
 1;
 
 
@@ -365,7 +400,7 @@ Kjetil Kjernsmo, C<< <kjetilk@cpan.org> >>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017 Kjetil Kjernsmo
+Copyright 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018 Kjetil Kjernsmo
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
